//============================================================================//
//
//  Software Name : MEGADOCK Version 2.1
//
//  Class Name : Control
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#ifndef Control_h
#define Control_h 1

#include "cpu_time.h"
#include "parallel.h"
#include "parameter.h"
#include "receptor.h"
#include "ligand.h"
#include "header_db.h"
#include "docking.h"

using namespace std;

class Control
{
 private:
  Control(Control &c){}
  const Control & operator=(const Control &c);
  CPUTime		*_cputime;
  Parallel		*_parallel;
  Parameter		*_parameter;
  Docking		*_docking;
 protected:
  virtual void		gridtable(int &ngrid,vector<int> &ngrid_table);
  virtual void		autogridr(Receptor &ipreceptor,const int &ngrid,
				  vector<int> &ngrid_table);
  virtual void		autogridl(Ligand &ipligand,const int &ngrid,
				  vector<int> &ngrid_table);
  virtual void		checkgridr(Receptor &ipreceptor);
  virtual void		checkgridl(Ligand &ipligand);
 public:
  Control(CPUTime *pcputime,Parallel *pparallel)
    : _cputime(pcputime),_parallel(pparallel) {
#ifdef DEBUG
    cout << "Constructing Control.\n";
#endif
  }
  virtual ~Control() {
#ifdef DEBUG
    cout << "Destructing Control.\n";
#endif
  }
  virtual void		initialize(int argc,char *argv[]);
  virtual void		execute();
};

#endif
