//============================================================================//
//
//  Software Name : MEGADOCK Version 2.1
//
//  Class Name : FFTProcess
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#ifndef FFTProcess_h
#define FFTProcess_h 1

#include <algorithm>
#include <functional>

#include "cpu_time.h"
#include "parallel.h"
#include "parameter.h"
#include "protein.h"
#include "fftw3.h"

using namespace std;

typedef struct {
  float score;
  int	index[4];
} SortScore;

class FFTProcess
{
 private:
  FFTProcess(FFTProcess &c) {}
  const FFTProcess & operator=(const FFTProcess &c);
  fftwf_complex			*_FFTWin;
  fftwf_complex			*_FFTWout;
  CPUTime			*_cputime;
  Parallel			*_parallel;
  Parameter			*_parameter;
  vector< vector<SortScore> >	_Top;
  vector< vector<SortScore> >	_Select;
  int				_Num_fft;
  float				*_FFT_rec_r;
  float				*_FFT_rec_i;
  float				**_FFT_lig_r;
  float				**_FFT_lig_i;
  int				*_Rotation;
 protected:
  virtual void			fft3d(const float &theta,int myid2);
 public:
  FFTProcess(CPUTime *pcputime,Parallel *pparallel,Parameter *pparameter)
    : _cputime(pcputime),_parallel(pparallel),_parameter(pparameter) {
#ifdef DEBUG
    cout << "Constructing FFTProcess.\n";
#endif
  }
  virtual ~FFTProcess() {
#ifdef DEBUG
    cout << "Destructing FFTProcess.\n";
#endif
  }
  virtual void		alloc_array(const int &maxlist,const int &maxfft,
				    const int &maxarray);
  virtual void		delete_array();
  virtual void		delete_fft();
  virtual void		receptor_fft(const int &ilist_rec,float *grid_r,
				     float *grid_i);
  virtual void		ligand_fft(float *grid_r,float *grid_i,int myid2);
  virtual void		alloc_fft(const int &maxfft);
  virtual void		fftinl(ifstream &fftlig,const int &im);
  virtual void		fftoutl(ofstream &fftlig);
  virtual void		fft_score(const int &irec,int myid2);
  virtual void		score_sort(const int &irec,int myid2);
  virtual void		top_score_clean(const int &nlist_rec);
  virtual int		num_fft() { return _Num_fft; }
  virtual void		num_fft(const int &i) { _Num_fft = i; }
  virtual void		rotation_index(int i,const int &j) { _Rotation[i] = j; }
  virtual float		top_score(const int &i,const int &j) {
    return _Top[i][j].score;
  }
  virtual int		top_index(int &i,const int &j,int k) {
    return _Top[i][j].index[k];
  }
  virtual void		sort_index(const int &nlist_rec,float *fwork,int *iwork);
};

#endif
