//============================================================================//
//
//  Software Name : MEGADOCK Version 2.1
//
//  Class Name : HeaderDB
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#include "header_db.h"

//============================================================================//
void HeaderDB::fft_read(const int &ilig)
//============================================================================//
{
  ifstream	fftligin;
  string	lfile;
  float		bsize;
  int		nangle;

#ifdef CHARACTER
  fftligin.open(_parameter->_LigFFT_file[ilig].c_str(),ios::in);
  if( !fftligin ) {
    cerr << "[ERROR]FFT file [" << _parameter->_LigFFT_file[ilig]
	 << "] was not open!!" << endl;
    exit(1);
  }

  fftligin >> _Num_fft >> bsize >> nangle;
  fftligin >> _PDB_file >> _Center[0] >> _Center[1] >> _Center[2];
#else
  fftligin.open(_parameter->_LigFFT_file[ilig].c_str());
  if( !fftligin ) {
    cerr << "[ERROR]FFT file [" << _parameter->_LigFFT_file[ilig]
	 << "] was not open!!" << endl;
    exit(1);
  }

  fftligin.read((char*)&_Num_fft,sizeof(int));
  fftligin.read((char*)&bsize,sizeof(float));
  fftligin.read((char*)&nangle,sizeof(int));

  int	csize;
  char	cfile[256];

  fftligin.read((char*)&csize,sizeof(int));
  fftligin.read(cfile,csize);
  _PDB_file = string(cfile);

  fftligin.read((char*)_Center,sizeof(float)*3);
#endif

  fftligin.close();

  if( nangle != _parameter->_NZangle ) {
    cerr << "[ERROR] Number of angle rotations is not consistent with ["
	 << _parameter->_LigFFT_file[ilig] << "]!!\n";
    exit(1);
  }

  if( fabs(_parameter->_Grid_size-bsize) > 1.0e-5 ) {
    cerr << "[ERROR] Voxel size is not consistent with ["
	 << _parameter->_LigFFT_file[ilig] << "]!!\n";
    exit(1);
  }
      
  return;
}

//============================================================================//
void HeaderDB::fft_comm(const int &ip,const int &nproc,const int &myid)
//============================================================================//
{
#ifdef PARALLEL
  const int	iread = ip % nproc;

  MPI_Bcast(&_Num_fft,1,MPI_INT,iread,MPI_COMM_WORLD);

  int	csize;
  char	cfile[256];

  if( myid == iread ) {
    csize = _PDB_file.size() + 1;
    strncpy(cfile,_PDB_file.c_str(),csize);
  }

  MPI_Bcast(&csize,1,MPI_INT,iread,MPI_COMM_WORLD);
  MPI_Bcast(cfile,csize,MPI_CHAR,iread,MPI_COMM_WORLD);

  if( myid != iread ) {
    _PDB_file = string(cfile);
  }

  MPI_Bcast(_Center,3,MPI_FLOAT,iread,MPI_COMM_WORLD);
#endif

  return;
}
