//============================================================================//
//
//  Software Name : MEGADOCK Version 2.1
//
//  Class Name : (main)
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#include "cpu_time.h"
#include "control.h"

#define VERSION "2.1"
#define LASTUPDATED "2011/11/24"

//============================================================================//
int main(int argc,char *argv[])
//============================================================================//
{
  Parallel	*_parallel;
  CPUTime	*_cputime;
  Control	*_control;

  struct timeval et1, et2;
  time_t 	ct1, ct2;
  int 		nproc, nproc2, myid;

  cout << " MEGADOCK ver. "<< VERSION << endl;
  cout << "      megadock@bi.cs.titech.ac.jp   lastupdated: " << LASTUPDATED << endl;
  cout << endl;

  gettimeofday(&et1,NULL);
  time(&ct1);

#ifdef PARALLEL
  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&nproc);
  MPI_Comm_rank(MPI_COMM_WORLD,&myid);

  if(myid == 0){
    cout << "#Using MPI parallelization: " << nproc << " processes." << endl;
  }
  for( int i = 0 ; i < nproc ; i++ ) {
    //if( i == myid ) { cout << "nproc(MPI parallel) = " << nproc << " myid = " << myid << endl; }
    MPI_Barrier(MPI_COMM_WORLD);
  }
#else
  nproc = 1;
  myid = 0;
#endif

#ifdef _OPENMP
  if( myid == 0 ) {
#pragma omp parallel
    {
      nproc2 = omp_get_num_threads();
      if(omp_get_thread_num() == 0){
	cout << "#Using OpenMP parallelization: " << nproc2 << " threads." << endl;
      }
      //#pragma omp critical
      //{
      //cout << "nproc2(OpenMP parallel) = " << nproc2 << " myid2 = " << omp_get_thread_num() << endl;
      //}
    }
  }
  else {
#pragma omp parallel
    {
      nproc2 = omp_get_num_threads();
    }
  }
#else
  nproc2 = 1;
#endif

  _cputime 	= new CPUTime();
  _cputime->initialize();

  _parallel 	= new Parallel(nproc,nproc2,myid);

  _control 	= new Control(_cputime,_parallel);
  _control->initialize(argc,argv);

  _control->execute();

  delete	_control;
  delete	_parallel;

#ifdef PARALLEL
  for( int i = 0 ; i < nproc ; i++ ) {
    if( i == myid ) {
      //cout << "\nmyid = " << myid;
      _cputime->output();
    }
    MPI_Barrier(MPI_COMM_WORLD);
  }
#else
  _cputime->output();
#endif

  delete	_cputime;

#ifdef PARALLEL
  MPI_Finalize();
#endif

  time(&ct2);
  gettimeofday(&et2,NULL);

  if( !myid ) {
    cout << "\nCPU time      = " << difftime(ct2,ct1) << " sec.\n";
    cout << "Elapsed time  = " << et2.tv_sec-et1.tv_sec
      + (float)((et2.tv_usec-et1.tv_usec)*1e-6) << " sec.\n";
  }

  return 0;
}
