//============================================================================//
//
//  Software Name : MEGADOCK Version 2.1
//
//  Class Name : Parameter
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#ifndef Parameter_h
#define Parameter_h 1

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <map>
#include <cassert>
#include <unistd.h>

#include "constant.h"
#include "parallel.h"

using namespace std;

class Parameter
{
 private:
  friend class			Control;
  friend class			Docking;
  friend class			FFTProcess;
  friend class			Protein;
  friend class			HeaderDB;
  Parameter(Parameter &c){}
  const Parameter & operator=(const Parameter &c);
  Parallel			*_parallel;
  string			_Parameter_file;
  string			_IO_file[3];
  vector<string>		_RecPDB_file;
  vector<string>		_LigPDB_file;
  vector<string>		_LigFFT_file;
  vector<string>		_RLOut_file;
  int				_Cal_step;
  int				_IO_flag[3];
  int				_Num_grid;
  int				_Num_fft;
  int				_Num_fft_flag;
  int				_Num_fft_l;
  int				_Num_rec;
  int				_Num_lig;
  int				_Num_output;
  int				_Num_output_flag;
  int				_Score_func;
  int				_Num_sort;
  float				_Elec_ratio;
  float				_Grid_size;
  float				_Delta_angle;
  float				_Grid_space_rec;
  float				_Grid_space_lig;
  float				*_Zangle;
  int				_NZangle;
  map<string,float>		_FTDockc;
  map<string,float>		_Charmmr;
  map<string,float>		_Charmmc;
 protected:
  virtual void			usage();
  virtual void			default_param();
  virtual void			pdb_step(const int &ipar);
  virtual void			pdbfft_step(const int &ipar);
  virtual void			fft_step(const int &ipar);
  virtual void			parameter_read();
  virtual void			pdb_filename();
  virtual void			pdbfft_filename();
  virtual void			fft_filename();
  virtual void			parameter_set();
  virtual void			ftdock_charge();
  virtual void			charmm_radius();
  virtual void			charmm_charge();
  virtual void			dangle15();
  virtual void			dangle6();
 public:
  Parameter(Parallel *pparallel) : _parallel(pparallel) {
#ifdef DEBUG
    cout << "Constructing Parameter.\n";
#endif
  }
  virtual			~Parameter() {
#ifdef DEBUG
    cout << "Destructing Parameter.\n";
#endif
  }
  virtual void			initialize(int argc,char *argv[]);
  virtual float			atom_radius(const string &atype);
  virtual float			atom_charge(const string &atype);
  virtual float			atom_ftcharge(const string &atype);

  virtual float                 grid_size(){ return _Grid_size; } //t.ishida
};

#endif
