//============================================================================//
//
//  Software Name : MEGADOCK Version 2.1
//
//  Class Name : PDBEntry
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#include "pdb_entry.h"

//============================================================================//
void PDBEntry::pdb_read()
//============================================================================//
{
  string	pdb, atomtype;
  vector<float>	coordinate0[3];

#ifdef DEBUG
  cout << "Input file = " << _Input_file << endl;
#endif

  ifstream Input(_Input_file.c_str(),ios::in);
  if( !Input ) {
    cerr << "[ERROR] PDB file [" << _Input_file << "] not open!!" << endl;
    exit(1);
  }

  while(1) {
    if( !getline(Input,pdb) ) break;
#ifdef DEBUG2
    cout << pdb << endl;
#endif

    if( pdb.substr(0,6) == "ATOM  " || pdb.substr(0,6) == "HETATM" ) {
      coordinate0[0].push_back(atof(pdb.substr(30,8).c_str()));
      coordinate0[1].push_back(atof(pdb.substr(38,8).c_str()));
      coordinate0[2].push_back(atof(pdb.substr(46,8).c_str()));

      atomtype = pdb.substr(11,10);
      _Atom_type.push_back(erase_space(atomtype));
    }
  }

  Input.close();

  _Num_atoms = coordinate0[0].size();

  _Coordinate = new float[3*_Num_atoms];
  if( !_Coordinate ) {
    cerr << "[ERROR] Out of memory. Number of atomic coordinates*3(x,y,z) = ("
	 << 3*_Num_atoms << ") for (_Coordinate) in pdb_entry.cpp!!\n";
    exit(1);
  }

  for( int i = 0 ; i < _Num_atoms ; i++ ) {
    float coord[3];

    for( int j = 0 ; j < 3 ; j++ ) {
      coord[j] = coordinate0[j][i];
    }
    coordinate(i,coord);
  }

  return;
}

//============================================================================//
string PDBEntry::erase_space(const string &s0)
//============================================================================//
{
  int 		n;
  string	s1;

  s1 = s0;

  while(1) {
    n = s1.find("  ");

    if( n == std::string::npos ) {
      break;
    }
    else {
      s1.erase(n,1);
    }
  }

  if( s1.substr(0,1) == " " ) {
    s1.erase(0,1);
  }

  if( s1.substr(s1.size()-1,1) == " " ) {
    s1.erase(s1.size()-1,1);
  }

  return s1;
}

//============================================================================//
void PDBEntry::pdb_comm(const int &ip,const int &nproc,const int &myid)
//============================================================================//
{
#ifdef PARALLEL
  const int	D1 = 8;
  const int	iread = ip % nproc;

  MPI_Bcast(&_Num_atoms,1,MPI_INT,iread,MPI_COMM_WORLD);

  int 	*bsize = new int[_Num_atoms];
  if( !bsize ) {
    cerr << "[ERROR] Out of memory. Number of atoms = ("
	 << _Num_atoms << ") for (bsize) in pdb_entry.cpp!!\n";
    MPI_Abort(MPI_COMM_WORLD,1);
  }

  char 	*buff = new char[D1*_Num_atoms];
  if( !buff ) {
    cerr << "[ERROR] Out of memory. Number of atoms x D1 = ("
	 << D1*_Num_atoms << ") for (buff) in pdb_entry.cpp!!\n";
    MPI_Abort(MPI_COMM_WORLD,1);
  }
  
  for( int i = 0 ; i < _Num_atoms ; i++ ) {
    if( myid == iread ) {
      bsize[i] = _Atom_type[i].size() + 1;
      strncpy(&buff[D1*i],_Atom_type[i].c_str(),bsize[i]);

#ifdef DEBUG2
      cout << "bsize[" << i << "] = " << bsize[i] << endl;
      cout << "&buff[" << D1*i << "] = " << &buff[D1*i] << endl;
#endif
    }
  }

  MPI_Bcast(bsize,_Num_atoms,MPI_INT,iread,MPI_COMM_WORLD);
  MPI_Bcast(buff,D1*_Num_atoms,MPI_CHAR,iread,MPI_COMM_WORLD);

  if( myid != iread ) {
    char buff2[D1];

    for( int i = 0 ; i < _Num_atoms ; i++ ) {
      strncpy(buff2,&buff[D1*i],bsize[i]);
      _Atom_type.push_back(string(buff2));
    }
  }

  float *cod = new float[_Num_atoms*3];
  if( !cod ) {
    cerr << "[ERROR] Out of memory. Number of atoms = ("
	 << _Num_atoms << ") for (cod) in pdb_entry.cpp!!\n";
    MPI_Abort(MPI_COMM_WORLD,1);
  }

  if( iread == myid ) {
    for( int i = 0 ; i < _Num_atoms*3 ; i++ ) {
      cod[i] = _Coordinate[i];
    }
  }

  MPI_Bcast(cod,_Num_atoms*3,MPI_FLOAT,iread,MPI_COMM_WORLD);

  if( myid != iread ) {
    _Coordinate = new float[3*_Num_atoms];
    if( !_Coordinate ) {
      cerr << "[ERROR] Out of memory. Number of atomic coordinates*3(x,y,z) = ("
	   << 3*_Num_atoms << ") for (_Coordinate) in pdb_entry.cpp!!\n";
      exit(1);
    }

    coord_all(cod);
  }

  delete [] bsize;
  delete [] buff;
  delete [] cod;
#endif

  return;
}
