//============================================================================//
//
//  Software Name : MEGADOCK Version 2.2
//
//  Class Name : Docking
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#ifndef Docking_h
#define Docking_h 1

#include "cpu_time.h"
#include "parallel.h"
#include "parameter.h"
#include "receptor.h"
#include "ligand.h"
#include "header_db.h"
#include "fft_process.h"

using namespace std;

class Docking
{
 private:
  Docking(Docking &c){}
  const Docking & operator=(const Docking &c);
  CPUTime		*_cputime;
  Parallel		*_parallel;
  Parameter		*_parameter;
  vector<Receptor>	_receptor;
  vector<Ligand>	_ligand;
  vector<HeaderDB>	_header;
  FFTProcess		*_fft_process;
  vector<int>		_FFT_list;
  vector< vector<int> >	_Rec_list;
  vector< vector<int> >	_Lig_list;
  float			*_Grid_coord;
  float			**_Mol_coord;
  float			*_Fwork;
  int			*_Iwork;
  int			_Memfw;
  int			_Memiw;
 protected:
  virtual void		create_list();
  virtual void		maxsize_voxel();
  virtual void		alloc_array(const int &maxatom,const int &num_grid,
				    const int &nag,const int &ng3);
  virtual void		create_voxel(Protein &rprotein,const int &num_grid,
				     int myid2);
  virtual void		ligand_rotationz(int ilig,float *theta,int myid2);
 public:
  Docking(CPUTime *pcputime,Parallel *pparallel,Parameter *pparameter,
	  vector<Receptor> &rreceptor,vector<Ligand> &rligand,
	  vector<HeaderDB> &rheader)
    : _cputime(pcputime),_parallel(pparallel),_parameter(pparameter),
    _receptor(rreceptor),_ligand(rligand),_header(rheader) {
#ifdef DEBUG
    cout << "Constructing Docking.\n";
#endif
  }
  virtual ~Docking() {
#ifdef DEBUG
    cout << "Destructing Docking.\n";
#endif
    delete [] _Grid_coord;
    delete [] _Fwork;
    delete [] _Iwork;
  }
  virtual void		initialize();
  virtual void		rec_init(const int &ilist);
  virtual void		dockz(const int &ilist,const int &ilist_lig);
  virtual void		output(const int &ilist,int ilist_lig);
  virtual int		fft_list(int &i) 	{ return _FFT_list[i]; }
  virtual int		fft_nlist() 		{ return _FFT_list.size(); }
  virtual int		rec_list(int &i,int &j) { return _Rec_list[i][j]; }
  virtual int		rec_nlist(int &i) 	{ return _Rec_list[i].size(); }
  virtual int		lig_list(int &i,int &j) { return _Lig_list[i][j]; }
  virtual int		lig_nlist(int &i) 	{ return _Lig_list[i].size(); }
  virtual void		dockz1(const int &ilig);
  virtual void		dock2(const int &ilist,const int &ilist_lig);
  virtual void		delete_class()	{ delete _fft_process; return; }
  virtual void		delete_fft()	{ _fft_process->delete_fft(); return; }
  virtual void		delete_array()	{ _fft_process->delete_array(); return; }
};

#endif
