//============================================================================//
//
//  Software Name : MEGADOCK Version 2.1
//
//  Class Name : Ligand
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#include "ligand.h"

//============================================================================//
void Ligand::katchalski(const int &score_func,const int &num_grid,
			float *grid_coord,float *mol_coord,float *fwork)
//============================================================================//
{
  const float lig_inside = 1.0;

  katchalski_base(num_grid,lig_inside,grid_coord,mol_coord,fwork);

  return;
}

//============================================================================//
void Ligand::gsc(const int &num_grid,float *grid_coord,float *mol_coord,
		 int *iwork,float *fwork)
//============================================================================//
{
  const	float	delta = 1.0;			// MEGADOCK parameter

  int		*nearesta;
  int		*surf_atom;
  float		*grid_r, *grid_i;
  float		*rjudge2, *radius_core2, *radius_surf2;
  float		*distance3;
  float		*xd, *yd, *zd;

  const int	na  = num_atoms();
  const int	nag = num_atoms() * num_grid;
  const int	ng2 = num_grid * num_grid;
  const int	ng3 = num_grid * num_grid * num_grid;

  nearesta 	= iwork;
  surf_atom	= &iwork[ng3];	
  grid_r	= fwork;
  grid_i	= &fwork[ng3];
  rjudge2 	= &fwork[ng3*2];
  radius_core2  = &fwork[ng3*2+na];
  radius_surf2  = &fwork[ng3*2+na*2];
  distance3     = &fwork[ng3*2+na*3];
  xd		= &fwork[ng3*3+na*3];
  yd		= &fwork[ng3*3+na*3+nag];
  zd		= &fwork[ng3*3+na*3+nag*2];

  precalc(num_grid,nearesta,rjudge2,radius_core2,radius_surf2);
  voxel_init(num_grid,grid_coord,mol_coord,grid_r,grid_i,distance3,xd,yd,zd);

  for( int l = 0 ; l < na ; l++ ) {
    const int lc = num_grid * l;

    for( int i = 0 ; i < num_grid ; i++ ) {
      if( xd[lc+i] > radius_core2[l] ) continue;
      const int ib = ng2 * i;

      for( int j = 0 ; j < num_grid ; j++ ) {
	if( yd[lc+j] > radius_core2[l] ) continue;
	const float	d2 = xd[lc+i] + yd[lc+j];
	const int	jb = ib + num_grid * j;

	for( int k = 0 ; k < num_grid ; k++ ) {
	  const float d3 = d2 + zd[lc+k];
	  if( d3 > radius_core2[l] ) continue;
	  const int kb = jb + k;

	  if( d3 < distance3[kb] ) {
	    distance3[kb] = d3;
	    nearesta[kb] = l;
	  }
	}
      }
    }
  }

  for( int i = 0 ; i < ng3 ; i++ ) {
    if( nearesta[i] == -99 ) continue;

    if( distance3[i] <= radius_core2[nearesta[i]] ) {
      grid_i[i] = 9.0 * delta;
    }
  }

  const int ng1 = num_grid;
  int	nsurf = 0;

  for( int i = 1 ; i < num_grid-1 ; i++ ) {
    const int ib = ng2 * i;

    for( int j = 1 ; j < num_grid-1 ; j++ ) {
      const int	jb = ib + num_grid * j;

      for( int k = 1 ; k < num_grid-1 ; k++ ) {
	const int kb = jb + k;

	if( grid_i[kb] == 9.0*delta ) {
	  if( grid_i[kb-ng2] == 0.0 || grid_i[kb+ng2] == 0.0 ||
	      grid_i[kb-ng1] == 0.0 || grid_i[kb+ng1] == 0.0 ||
	      grid_i[kb-1  ] == 0.0 || grid_i[kb+1  ] == 0.0 ) {
	    grid_i[kb] = -9999.0;

	    int l;

	    for( l = 0 ; l < nsurf ; l++ ) {
	      if( surf_atom[l] == nearesta[kb] ) {
		break;
	      }
	    }

	    if( l >= nsurf ) {
	      surf_atom[nsurf++] = nearesta[kb];
	    }
	  }
	}
      }
    }
  }

  for( int i = ng2 ; i < ng3-1 ; i++ ) {
    if( grid_i[i] == -9999.0 ) {
      grid_i[i] = 0.0;
    }
  }

  for( int l = 0 ; l < nsurf ; l++ ) {
    const int lc = num_grid * surf_atom[l];

    for( int i = 0 ; i < num_grid ; i++ ) {
      if( xd[lc+i] > radius_surf2[surf_atom[l]] ) continue;
      const int ib = ng2 * i;

      for( int j = 0 ; j < num_grid ; j++ ) {
	if( yd[lc+j] > radius_surf2[surf_atom[l]] ) continue;
	const float	d2 = xd[lc+i] + yd[lc+j];
	const int	jb = ib + num_grid * j;

	for( int k = 0 ; k < num_grid ; k++ ) {
	  if( d2+zd[lc+k] <= radius_surf2[surf_atom[l]] ) {
	    int	kb = jb + k;
	    grid_r[kb] = 1.0;
	    grid_i[kb] = 0.0;
	  }
	}
      }
    }
  }

  return;
}

//============================================================================//
void Ligand::psc(const int &num_grid,float *grid_coord,float *mol_coord,
		 int *iwork,float *fwork)
//============================================================================//
{
  const	float	delta = 1.0;			// MEGADOCK parameter

  int		*nearesta;
  int		*surf_atom;
  int		*ijk;
  float		*grid_r, *grid_i;
  float		*rjudge2, *radius_core2, *radius_surf2;
  float		*distance3;
  float		*xd, *yd, *zd;

  const int	na  = num_atoms();
  const int	nag = num_atoms() * num_grid;
  const int	ng2 = num_grid * num_grid;
  const int	ng3 = num_grid * num_grid * num_grid;

  nearesta 	= iwork;
  surf_atom	= &iwork[ng3];	
  ijk		= &iwork[ng3*2+na];	
  grid_r	= fwork;
  grid_i	= &fwork[ng3];
  rjudge2 	= &fwork[ng3*2];
  radius_core2  = &fwork[ng3*2+na];
  radius_surf2  = &fwork[ng3*2+na*2];
  distance3     = &fwork[ng3*2+na*3];
  xd		= &fwork[ng3*3+na*3];
  yd		= &fwork[ng3*3+na*3+nag];
  zd		= &fwork[ng3*3+na*3+nag*2];

  precalc(num_grid,nearesta,rjudge2,radius_core2,radius_surf2);
  voxel_init(num_grid,grid_coord,mol_coord,grid_r,grid_i,distance3,xd,yd,zd);

  for( int l = 0 ; l < na ; l++ ) {
    const int lc = num_grid * l;
    float distancea = BIG;

    for( int i = 0 ; i < num_grid ; i++ ) {
      if( xd[lc+i] > distancea ) continue;

      for( int j = 0 ; j < num_grid ; j++ ) {
	if( yd[lc+j] > distancea ) continue;
	const float	d2 = xd[lc+i] + yd[lc+j];

	for( int k = 0 ; k < num_grid ; k++ ) {
	  const float d3 = d2 + zd[lc+k];

	  if( d3 < distancea ) {
	    distancea = d3;
	    ijk[3*l  ] = i;
	    ijk[3*l+1] = j;
	    ijk[3*l+2] = k;
	  }
	}
      }
    }
  }

  for( int l = 0 ; l < na ; l++ ) {
    const int i = ijk[3*l  ];
    const int j = ijk[3*l+1];
    const int k = ijk[3*l+2];

    grid_r[i*ng2 + j*num_grid + k] = 1.0;
  }

  for( int l = 0 ; l < na ; l++ ) {
    const int lc = num_grid * l;

    for( int i = 0 ; i < num_grid ; i++ ) {
      if( xd[lc+i] > radius_core2[l] ) continue;
      const int ib = ng2 * i;

      for( int j = 0 ; j < num_grid ; j++ ) {
	if( yd[lc+j] > radius_core2[l] ) continue;
	const float	d2 = xd[lc+i] + yd[lc+j];
	const int	jb = ib + num_grid * j;

	for( int k = 0 ; k < num_grid ; k++ ) {
	  const float 	d3 = d2 + zd[lc+k];
	  const int	kb = jb + k;

	  if( d3 < distance3[kb] ) {
	    distance3[kb] = d3;
	    nearesta[kb] = l;
	  }
	}
      }
    }
  }

  for( int i = 0 ; i < ng3 ; i++ ) {
    if( nearesta[i] == -99 ) continue;

    if( distance3[i] <= radius_core2[nearesta[i]] ) {
      grid_i[i] = 9.0 * delta;
    }
  }

  const int ng1 = num_grid;
  int	nsurf = 0;

  for( int i = 1 ; i < num_grid-1 ; i++ ) {
    const int ib = ng2 * i;

    for( int j = 1 ; j < num_grid-1 ; j++ ) {
      const int	jb = ib + num_grid * j;

      for( int k = 1 ; k < num_grid-1 ; k++ ) {
	const int kb = jb + k;

	if( grid_i[kb] == 9.0*delta ) {
	  if( grid_i[kb-ng2] == 0.0 || grid_i[kb+ng2] == 0.0 ||
	      grid_i[kb-ng1] == 0.0 || grid_i[kb+ng1] == 0.0 ||
	      grid_i[kb-1  ] == 0.0 || grid_i[kb+1  ] == 0.0 ) {
	    grid_i[kb] = -9999.0;

	    int l;

	    for( l = 0 ; l < nsurf ; l++ ) {
	      if( surf_atom[l] == nearesta[kb] ) {
		break;
	      }
	    }

	    if( l >= nsurf ) {
	      surf_atom[nsurf++] = nearesta[kb];
	    }
	  }
	}
      }
    }
  }

  for( int i = ng2 ; i < ng3-1 ; i++ ) {
    if( grid_i[i] == -9999.0 ) {
      grid_i[i] = 0.0;
    }
  }

  for( int l = 0 ; l < nsurf ; l++ ) {
    const int lc = num_grid * surf_atom[l];

    for( int i = 0 ; i < num_grid ; i++ ) {
      if( xd[lc+i] > radius_surf2[surf_atom[l]] ) continue;
      const int ib = ng2 * i;

      for( int j = 0 ; j < num_grid ; j++ ) {
	if( yd[lc+j] > radius_surf2[surf_atom[l]] ) continue;
	const float	d2 = xd[lc+i] + yd[lc+j];
	const int	jb = ib + num_grid * j;

	for( int k = 0 ; k < num_grid ; k++ ) {
	  const float d3 = d2 + zd[lc+k];

	  if( d3 <= radius_surf2[surf_atom[l]] ) {
	    grid_i[jb+k] = 3.0 * delta;
	  }
	}
      }
    }
  }

  return;
}

//============================================================================//
void Ligand::rpsc(const int &num_grid,float *grid_coord,float *mol_coord,
		  int *iwork,float *fwork)
//============================================================================//
{
  const float	delta = 2.0;			// MEGADOCK parameter

  int		*nearesta;
  int		*surf_atom;
  float		*grid_r, *grid_i;
  float		*rjudge2, *radius_core2, *radius_surf2;
  float		*distance3;
  float		*xd, *yd, *zd;

  const int	na  = num_atoms();
  const int	nag = num_atoms() * num_grid;
  const int	ng2 = num_grid * num_grid;
  const int	ng3 = num_grid * num_grid * num_grid;

  nearesta 	= iwork;
  surf_atom	= &iwork[ng3];	
  grid_r	= fwork;
  grid_i	= &fwork[ng3];
  rjudge2 	= &fwork[ng3*2];
  radius_core2  = &fwork[ng3*2+na];
  radius_surf2  = &fwork[ng3*2+na*2];
  distance3     = &fwork[ng3*2+na*3];
  xd		= &fwork[ng3*3+na*3];
  yd		= &fwork[ng3*3+na*3+nag];
  zd		= &fwork[ng3*3+na*3+nag*2];

  precalc(num_grid,nearesta,rjudge2,radius_core2,radius_surf2);
  voxel_init(num_grid,grid_coord,mol_coord,grid_r,grid_i,distance3,xd,yd,zd);

  for( int l = 0 ; l < na ; l++ ) {
    const int lc = num_grid * l;

    for( int i = 0 ; i < num_grid ; i++ ) {
      if( xd[lc+i] > radius_core2[l] ) continue;
      const int ib = ng2 * i;

      for( int j = 0 ; j < num_grid ; j++ ) {
	if( yd[lc+j] > radius_core2[l] ) continue;
	const float	d2 = xd[lc+i] + yd[lc+j];
	const int	jb = ib + num_grid * j;

	for( int k = 0 ; k < num_grid ; k++ ) {
	  const float	d3 = d2 + zd[lc+k];
	  const int	kb = jb + k;

	  if( d3 < distance3[kb] ) {
	    distance3[kb] = d3;
	    nearesta[kb] = l;
	  }
	}
      }
    }
  }

  for( int i = 0 ; i < ng3 ; i++ ) {
    if( nearesta[i] == -99 ) continue;

    if( distance3[i] <= radius_core2[nearesta[i]] ) {
      grid_r[i] = delta;
    }
  }

  const int ng1 = num_grid;
  int	nsurf = 0;

  for( int i = 1 ; i < num_grid-1 ; i++ ) {
    const int ib = ng2 * i;

    for( int j = 1 ; j < num_grid-1 ; j++ ) {
      const int	jb = ib + num_grid * j;

      for( int k = 1 ; k < num_grid-1 ; k++ ) {
	const int kb = jb + k;

	if( grid_r[kb] == delta ) {
	  if( grid_r[kb-ng2] == 0.0 || grid_r[kb+ng2] == 0.0 ||
	      grid_r[kb-ng1] == 0.0 || grid_r[kb+ng1] == 0.0 ||
	      grid_r[kb-1  ] == 0.0 || grid_r[kb+1  ] == 0.0 ) {
	    grid_r[kb] = -9999.0;

	    int l;

	    for( l = 0 ; l < nsurf ; l++ ) {
	      if( surf_atom[l] == nearesta[kb] ) {
		break;
	      }
	    }

	    if( l >= nsurf ) {
	      surf_atom[nsurf++] = nearesta[kb];
	    }
	  }
	}
      }
    }
  }

  for( int i = ng2 ; i < ng3-ng2 ; i++ ) {
    if( grid_r[i] == -9999.0 ) {
      grid_r[i] = 0.0;
    }
  }

  for( int l = 0 ; l < nsurf ; l++ ) {
    const int lc = num_grid * surf_atom[l];

    for( int i = 0 ; i < num_grid ; i++ ) {
      if( xd[lc+i] > radius_surf2[surf_atom[l]] ) continue;
      const int ib = ng2 * i;

      for( int j = 0 ; j < num_grid ; j++ ) {
	if( yd[lc+j] > radius_surf2[surf_atom[l]] ) continue;
	const float	d2 = xd[lc+i] + yd[lc+j];
	const int	jb = ib + num_grid * j;

	for( int k = 0 ; k < num_grid ; k++ ) {
	  const float d3 = d2 + zd[lc+k];

	  if( d3 <= radius_surf2[surf_atom[l]] ) {
	    grid_r[jb+k] = 1.0;
	  }
	}
      }
    }
  }

  return;
}

//============================================================================//
void Ligand::electro(const float &beta,const float &eratio,
		     const int &num_grid,float *grid_coord,float *mol_coord,
		     int *iwork, float *fwork, float grid_size)
//============================================================================//
{
  int		*ijk;
  float		*grid_i;
  const float	*xd, *yd, *zd;

  const int	na = num_atoms();;
  const int	nag = num_atoms() * num_grid;
  const int	ng2 = num_grid * num_grid;
  const int	ng3 = num_grid * num_grid * num_grid;

  ijk 		= &iwork[ng3*2+na];	
  grid_i	= &fwork[ng3];
  xd		= &fwork[ng3*3+na*3];
  yd		= &fwork[ng3*3+na*3+nag];
  zd		= &fwork[ng3*3+na*3+nag*2];

  //modified by t.ishida at 2011/4/15
  //const float           grid_size=rparameter->grid_size(); 
  const float           max_range=grid_size*1.5; 
  const float           voxel_range = grid_size/2.0;

  for( int l = 0 ; l < na ; l++ ) {
    const int lc = num_grid * l;
    float distancea = max_range;

    for( int i = 0 ; i < num_grid ; i++ ) {
      if( xd[lc+i] > voxel_range ) continue;

      for( int j = 0 ; j < num_grid ; j++ ) {
	if( yd[lc+j] > voxel_range ) continue;
	const float d2 = xd[lc+i] + yd[lc+j];

	for( int k = 0 ; k < num_grid ; k++ ) {
	  const float d3 = d2 + zd[lc+k];

	  if( d3 < distancea ) {
	    distancea 	= d3;
	    ijk[3*l  ] 	= i;
	    ijk[3*l+1] 	= j;
	    ijk[3*l+2] 	= k;
	  }
	}
      }
    }
  }

  for( int l = 0 ; l < na ; l++ ) {
    const int	i = ijk[3*l  ];
    const int	j = ijk[3*l+1];
    const int	k = ijk[3*l+2];

    grid_i[i*ng2+j*num_grid+k] = _Charge[l];
  }

  return;
}

//============================================================================//
void Ligand::precalc(const int &num_grid,int *nearesta,float *rjudge2,
		     float *radius_core2,float *radius_surf2)
//============================================================================//
{
  const	float	rcore2 = 1.5;			// ZDOCK parameter
  const	float	rsurf2 = 1.0;			// ZDOCK parameter

  const int	na  = num_atoms();
  const int	ng3 = num_grid * num_grid * num_grid;

  for( int i = 0 ; i < na ; i++ ) {
    radius_core2[i] = _Radius[i] * _Radius[i] * rcore2;
    radius_surf2[i] = _Radius[i] * _Radius[i] * rsurf2;
  }

  for( int i = 0 ; i < ng3 ; i++ ) {
    nearesta[i] = -99;
  }

  return;
}
