//============================================================================//
//
//  Software Name : MEGADOCK Version 2.1
//
//  Class Name : Protein
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#ifndef Protein_h
#define Protein_h 1

#include "pdb_entry.h"
#include "parameter.h"

using namespace std;

class Protein : public PDBEntry		// Protein class
{
 private:
  friend class		Docking;
  friend class		Receptor;
  friend class		Ligand;
  //  const Protein & operator=(const Protein &c);
  int			_Num_grid;
  float			_Edge[3][2];
  float			_Center[3];
  float			_Angle[3];
  float			*_Radius;
  float			*_Charge;
  float			*_Coord;
 protected:
  virtual void			axis_edge();
  virtual void 	katchalski_base(const int &num_grid,const float &mol_inside,
				float *grid_coord,float *mol_coord,
				float *fwork);
  virtual void	voxel_init(const int &num_grid,float *grid_coord,
			   float *mol_coord,float *grid_r,float *grid_i,
			   float *distance3,float *xd,float *yd,float *zd);
 public:
  Protein(string &rinput_file) : PDBEntry(rinput_file) {
#ifdef DEBUG
    cout << "Constructing Protein.\n";
#endif
  }
  virtual ~Protein() {
#ifdef DEBUG
    cout << "Destructing Protein.\n";
#endif
  }
  virtual void		initialize(Parameter *rparameter,const int &ip,
				   const int &nproc,const int &myid);
  virtual void		shift_center(float *mol_coord);
  virtual float		edge(const int &i,int j){ return _Edge[i][j]; }
  virtual float		center(const int &i)	{ return _Center[i]; }
  virtual void		center(float *coord) {
    for( int i = 0 ; i < 3 ; i++ ) {
      _Center[i] = coord[i];
    }
    return;
  }
  virtual float		angle(const int &i)	{ return _Angle[i]; }
  virtual void		angle(float *angle)	{
    for( int i = 0 ; i < 3 ; i++ ) {
      _Angle[i] = angle[i];
    }
    return;
  }
  virtual int		num_grid() 		{ return _Num_grid; }
  virtual void		num_grid(const int &num_grid) {
    _Num_grid = num_grid;
    return;
  }
  virtual void	katchalski(const int &score_func,const int &num_grid,
			   float *grid_coord,float *mol_coord,float *fwork) {;}
  virtual void	gsc(const int &num_grid,float *grid_coord,float *mol_coord,
		    int *iwork,float *fwork) {;}
  virtual void	psc(const int &num_grid,float *grid_coord,float *mol_coord,
		    int *iwork,float *fwork) {;}
  virtual void	rpsc(const int &num_grid,float *grid_coord,float *mol_coord,
		     int *iwork,float *fwork) {;}
  virtual void	electro(const float &beta,const float &eratio,
			const int &num_grid,float *grid_coord,float *mol_coord,
			int *iwork,float *fwork, float grid_size) {;}
  virtual void 	precalc(const int &num_grid,int *nearesta,float *rjudge2,
			float *radius_core2,float *radius_surf2) {;}
};

#endif
